# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetPartnerAttachmentResult',
    'AwaitableGetPartnerAttachmentResult',
    'get_partner_attachment',
    'get_partner_attachment_output',
]

@pulumi.output_type
class GetPartnerAttachmentResult:
    """
    A collection of values returned by getPartnerAttachment.
    """
    def __init__(__self__, bgp=None, childrens=None, connection_bandwidth_in_mbps=None, created_at=None, id=None, naas_provider=None, name=None, parent_uuid=None, redundancy_zone=None, region=None, state=None, vpc_ids=None):
        if bgp and not isinstance(bgp, dict):
            raise TypeError("Expected argument 'bgp' to be a dict")
        pulumi.set(__self__, "bgp", bgp)
        if childrens and not isinstance(childrens, list):
            raise TypeError("Expected argument 'childrens' to be a list")
        pulumi.set(__self__, "childrens", childrens)
        if connection_bandwidth_in_mbps and not isinstance(connection_bandwidth_in_mbps, int):
            raise TypeError("Expected argument 'connection_bandwidth_in_mbps' to be a int")
        pulumi.set(__self__, "connection_bandwidth_in_mbps", connection_bandwidth_in_mbps)
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if naas_provider and not isinstance(naas_provider, str):
            raise TypeError("Expected argument 'naas_provider' to be a str")
        pulumi.set(__self__, "naas_provider", naas_provider)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if parent_uuid and not isinstance(parent_uuid, str):
            raise TypeError("Expected argument 'parent_uuid' to be a str")
        pulumi.set(__self__, "parent_uuid", parent_uuid)
        if redundancy_zone and not isinstance(redundancy_zone, str):
            raise TypeError("Expected argument 'redundancy_zone' to be a str")
        pulumi.set(__self__, "redundancy_zone", redundancy_zone)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if vpc_ids and not isinstance(vpc_ids, list):
            raise TypeError("Expected argument 'vpc_ids' to be a list")
        pulumi.set(__self__, "vpc_ids", vpc_ids)

    @_builtins.property
    @pulumi.getter
    def bgp(self) -> Optional['outputs.GetPartnerAttachmentBgpResult']:
        return pulumi.get(self, "bgp")

    @_builtins.property
    @pulumi.getter
    def childrens(self) -> Sequence[_builtins.str]:
        return pulumi.get(self, "childrens")

    @_builtins.property
    @pulumi.getter(name="connectionBandwidthInMbps")
    def connection_bandwidth_in_mbps(self) -> _builtins.int:
        return pulumi.get(self, "connection_bandwidth_in_mbps")

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> _builtins.str:
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="naasProvider")
    def naas_provider(self) -> _builtins.str:
        return pulumi.get(self, "naas_provider")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="parentUuid")
    def parent_uuid(self) -> _builtins.str:
        return pulumi.get(self, "parent_uuid")

    @_builtins.property
    @pulumi.getter(name="redundancyZone")
    def redundancy_zone(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "redundancy_zone")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def state(self) -> _builtins.str:
        return pulumi.get(self, "state")

    @_builtins.property
    @pulumi.getter(name="vpcIds")
    def vpc_ids(self) -> Sequence[_builtins.str]:
        return pulumi.get(self, "vpc_ids")


class AwaitableGetPartnerAttachmentResult(GetPartnerAttachmentResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetPartnerAttachmentResult(
            bgp=self.bgp,
            childrens=self.childrens,
            connection_bandwidth_in_mbps=self.connection_bandwidth_in_mbps,
            created_at=self.created_at,
            id=self.id,
            naas_provider=self.naas_provider,
            name=self.name,
            parent_uuid=self.parent_uuid,
            redundancy_zone=self.redundancy_zone,
            region=self.region,
            state=self.state,
            vpc_ids=self.vpc_ids)


def get_partner_attachment(bgp: Optional[Union['GetPartnerAttachmentBgpArgs', 'GetPartnerAttachmentBgpArgsDict']] = None,
                           id: Optional[_builtins.str] = None,
                           name: Optional[_builtins.str] = None,
                           redundancy_zone: Optional[_builtins.str] = None,
                           opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetPartnerAttachmentResult:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['bgp'] = bgp
    __args__['id'] = id
    __args__['name'] = name
    __args__['redundancyZone'] = redundancy_zone
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getPartnerAttachment:getPartnerAttachment', __args__, opts=opts, typ=GetPartnerAttachmentResult).value

    return AwaitableGetPartnerAttachmentResult(
        bgp=pulumi.get(__ret__, 'bgp'),
        childrens=pulumi.get(__ret__, 'childrens'),
        connection_bandwidth_in_mbps=pulumi.get(__ret__, 'connection_bandwidth_in_mbps'),
        created_at=pulumi.get(__ret__, 'created_at'),
        id=pulumi.get(__ret__, 'id'),
        naas_provider=pulumi.get(__ret__, 'naas_provider'),
        name=pulumi.get(__ret__, 'name'),
        parent_uuid=pulumi.get(__ret__, 'parent_uuid'),
        redundancy_zone=pulumi.get(__ret__, 'redundancy_zone'),
        region=pulumi.get(__ret__, 'region'),
        state=pulumi.get(__ret__, 'state'),
        vpc_ids=pulumi.get(__ret__, 'vpc_ids'))
def get_partner_attachment_output(bgp: Optional[pulumi.Input[Optional[Union['GetPartnerAttachmentBgpArgs', 'GetPartnerAttachmentBgpArgsDict']]]] = None,
                                  id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                                  name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                                  redundancy_zone: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                                  opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetPartnerAttachmentResult]:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['bgp'] = bgp
    __args__['id'] = id
    __args__['name'] = name
    __args__['redundancyZone'] = redundancy_zone
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getPartnerAttachment:getPartnerAttachment', __args__, opts=opts, typ=GetPartnerAttachmentResult)
    return __ret__.apply(lambda __response__: GetPartnerAttachmentResult(
        bgp=pulumi.get(__response__, 'bgp'),
        childrens=pulumi.get(__response__, 'childrens'),
        connection_bandwidth_in_mbps=pulumi.get(__response__, 'connection_bandwidth_in_mbps'),
        created_at=pulumi.get(__response__, 'created_at'),
        id=pulumi.get(__response__, 'id'),
        naas_provider=pulumi.get(__response__, 'naas_provider'),
        name=pulumi.get(__response__, 'name'),
        parent_uuid=pulumi.get(__response__, 'parent_uuid'),
        redundancy_zone=pulumi.get(__response__, 'redundancy_zone'),
        region=pulumi.get(__response__, 'region'),
        state=pulumi.get(__response__, 'state'),
        vpc_ids=pulumi.get(__response__, 'vpc_ids')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetKubernetesVersionsResult',
    'AwaitableGetKubernetesVersionsResult',
    'get_kubernetes_versions',
    'get_kubernetes_versions_output',
]

@pulumi.output_type
class GetKubernetesVersionsResult:
    """
    A collection of values returned by getKubernetesVersions.
    """
    def __init__(__self__, id=None, latest_version=None, valid_versions=None, version_prefix=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if latest_version and not isinstance(latest_version, str):
            raise TypeError("Expected argument 'latest_version' to be a str")
        pulumi.set(__self__, "latest_version", latest_version)
        if valid_versions and not isinstance(valid_versions, list):
            raise TypeError("Expected argument 'valid_versions' to be a list")
        pulumi.set(__self__, "valid_versions", valid_versions)
        if version_prefix and not isinstance(version_prefix, str):
            raise TypeError("Expected argument 'version_prefix' to be a str")
        pulumi.set(__self__, "version_prefix", version_prefix)

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="latestVersion")
    def latest_version(self) -> _builtins.str:
        """
        The most recent version available.
        """
        return pulumi.get(self, "latest_version")

    @_builtins.property
    @pulumi.getter(name="validVersions")
    def valid_versions(self) -> Sequence[_builtins.str]:
        """
        A list of available versions.
        """
        return pulumi.get(self, "valid_versions")

    @_builtins.property
    @pulumi.getter(name="versionPrefix")
    def version_prefix(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "version_prefix")


class AwaitableGetKubernetesVersionsResult(GetKubernetesVersionsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetKubernetesVersionsResult(
            id=self.id,
            latest_version=self.latest_version,
            valid_versions=self.valid_versions,
            version_prefix=self.version_prefix)


def get_kubernetes_versions(version_prefix: Optional[_builtins.str] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetKubernetesVersionsResult:
    """
    Provides access to the available DigitalOcean Kubernetes Service versions.

    ## Example Usage

    ### Output a list of all available versions

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_kubernetes_versions()
    pulumi.export("k8s-versions", example.valid_versions)
    ```

    ### Create a Kubernetes cluster using the most recent version available

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_kubernetes_versions()
    example_cluster = digitalocean.KubernetesCluster("example-cluster",
        name="example-cluster",
        region=digitalocean.Region.LON1,
        version=example.latest_version,
        node_pool={
            "name": "default",
            "size": "s-1vcpu-2gb",
            "node_count": 3,
        })
    ```

    ### Pin a Kubernetes cluster to a specific minor version

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_kubernetes_versions(version_prefix="1.22.")
    example_cluster = digitalocean.KubernetesCluster("example-cluster",
        name="example-cluster",
        region=digitalocean.Region.LON1,
        version=example.latest_version,
        node_pool={
            "name": "default",
            "size": "s-1vcpu-2gb",
            "node_count": 3,
        })
    ```


    :param _builtins.str version_prefix: If provided, the provider will only return versions that match the string prefix. For example, `1.15.` will match all 1.15.x series releases.
    """
    __args__ = dict()
    __args__['versionPrefix'] = version_prefix
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getKubernetesVersions:getKubernetesVersions', __args__, opts=opts, typ=GetKubernetesVersionsResult).value

    return AwaitableGetKubernetesVersionsResult(
        id=pulumi.get(__ret__, 'id'),
        latest_version=pulumi.get(__ret__, 'latest_version'),
        valid_versions=pulumi.get(__ret__, 'valid_versions'),
        version_prefix=pulumi.get(__ret__, 'version_prefix'))
def get_kubernetes_versions_output(version_prefix: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                                   opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetKubernetesVersionsResult]:
    """
    Provides access to the available DigitalOcean Kubernetes Service versions.

    ## Example Usage

    ### Output a list of all available versions

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_kubernetes_versions()
    pulumi.export("k8s-versions", example.valid_versions)
    ```

    ### Create a Kubernetes cluster using the most recent version available

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_kubernetes_versions()
    example_cluster = digitalocean.KubernetesCluster("example-cluster",
        name="example-cluster",
        region=digitalocean.Region.LON1,
        version=example.latest_version,
        node_pool={
            "name": "default",
            "size": "s-1vcpu-2gb",
            "node_count": 3,
        })
    ```

    ### Pin a Kubernetes cluster to a specific minor version

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_kubernetes_versions(version_prefix="1.22.")
    example_cluster = digitalocean.KubernetesCluster("example-cluster",
        name="example-cluster",
        region=digitalocean.Region.LON1,
        version=example.latest_version,
        node_pool={
            "name": "default",
            "size": "s-1vcpu-2gb",
            "node_count": 3,
        })
    ```


    :param _builtins.str version_prefix: If provided, the provider will only return versions that match the string prefix. For example, `1.15.` will match all 1.15.x series releases.
    """
    __args__ = dict()
    __args__['versionPrefix'] = version_prefix
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getKubernetesVersions:getKubernetesVersions', __args__, opts=opts, typ=GetKubernetesVersionsResult)
    return __ret__.apply(lambda __response__: GetKubernetesVersionsResult(
        id=pulumi.get(__response__, 'id'),
        latest_version=pulumi.get(__response__, 'latest_version'),
        valid_versions=pulumi.get(__response__, 'valid_versions'),
        version_prefix=pulumi.get(__response__, 'version_prefix')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['PartnerAttachmentArgs', 'PartnerAttachment']

@pulumi.input_type
class PartnerAttachmentArgs:
    def __init__(__self__, *,
                 connection_bandwidth_in_mbps: pulumi.Input[_builtins.int],
                 naas_provider: pulumi.Input[_builtins.str],
                 region: pulumi.Input[_builtins.str],
                 vpc_ids: pulumi.Input[Sequence[pulumi.Input[_builtins.str]]],
                 bgp: Optional[pulumi.Input['PartnerAttachmentBgpArgs']] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 parent_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 redundancy_zone: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a PartnerAttachment resource.
        :param pulumi.Input[_builtins.int] connection_bandwidth_in_mbps: The connection bandwidth in Mbps
        :param pulumi.Input[_builtins.str] naas_provider: The NaaS provider
        :param pulumi.Input[_builtins.str] region: The region where the Partner Attachment will be created
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] vpc_ids: The list of VPC IDs to attach the Partner Attachment to
        :param pulumi.Input[_builtins.str] name: The name of the Partner Attachment
        :param pulumi.Input[_builtins.str] parent_uuid: The UUID of the Parent Partner Attachment
        :param pulumi.Input[_builtins.str] redundancy_zone: The redundancy zone for the NaaS
        """
        pulumi.set(__self__, "connection_bandwidth_in_mbps", connection_bandwidth_in_mbps)
        pulumi.set(__self__, "naas_provider", naas_provider)
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "vpc_ids", vpc_ids)
        if bgp is not None:
            pulumi.set(__self__, "bgp", bgp)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if parent_uuid is not None:
            pulumi.set(__self__, "parent_uuid", parent_uuid)
        if redundancy_zone is not None:
            pulumi.set(__self__, "redundancy_zone", redundancy_zone)

    @_builtins.property
    @pulumi.getter(name="connectionBandwidthInMbps")
    def connection_bandwidth_in_mbps(self) -> pulumi.Input[_builtins.int]:
        """
        The connection bandwidth in Mbps
        """
        return pulumi.get(self, "connection_bandwidth_in_mbps")

    @connection_bandwidth_in_mbps.setter
    def connection_bandwidth_in_mbps(self, value: pulumi.Input[_builtins.int]):
        pulumi.set(self, "connection_bandwidth_in_mbps", value)

    @_builtins.property
    @pulumi.getter(name="naasProvider")
    def naas_provider(self) -> pulumi.Input[_builtins.str]:
        """
        The NaaS provider
        """
        return pulumi.get(self, "naas_provider")

    @naas_provider.setter
    def naas_provider(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "naas_provider", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Input[_builtins.str]:
        """
        The region where the Partner Attachment will be created
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="vpcIds")
    def vpc_ids(self) -> pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]:
        """
        The list of VPC IDs to attach the Partner Attachment to
        """
        return pulumi.get(self, "vpc_ids")

    @vpc_ids.setter
    def vpc_ids(self, value: pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]):
        pulumi.set(self, "vpc_ids", value)

    @_builtins.property
    @pulumi.getter
    def bgp(self) -> Optional[pulumi.Input['PartnerAttachmentBgpArgs']]:
        return pulumi.get(self, "bgp")

    @bgp.setter
    def bgp(self, value: Optional[pulumi.Input['PartnerAttachmentBgpArgs']]):
        pulumi.set(self, "bgp", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the Partner Attachment
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="parentUuid")
    def parent_uuid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of the Parent Partner Attachment
        """
        return pulumi.get(self, "parent_uuid")

    @parent_uuid.setter
    def parent_uuid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "parent_uuid", value)

    @_builtins.property
    @pulumi.getter(name="redundancyZone")
    def redundancy_zone(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The redundancy zone for the NaaS
        """
        return pulumi.get(self, "redundancy_zone")

    @redundancy_zone.setter
    def redundancy_zone(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "redundancy_zone", value)


@pulumi.input_type
class _PartnerAttachmentState:
    def __init__(__self__, *,
                 bgp: Optional[pulumi.Input['PartnerAttachmentBgpArgs']] = None,
                 childrens: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 connection_bandwidth_in_mbps: Optional[pulumi.Input[_builtins.int]] = None,
                 created_at: Optional[pulumi.Input[_builtins.str]] = None,
                 naas_provider: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 parent_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 redundancy_zone: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 state: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        Input properties used for looking up and filtering PartnerAttachment resources.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] childrens: The children uuids of Partner Attachment
        :param pulumi.Input[_builtins.int] connection_bandwidth_in_mbps: The connection bandwidth in Mbps
        :param pulumi.Input[_builtins.str] created_at: The date and time when the Partner Attachment was created
        :param pulumi.Input[_builtins.str] naas_provider: The NaaS provider
        :param pulumi.Input[_builtins.str] name: The name of the Partner Attachment
        :param pulumi.Input[_builtins.str] parent_uuid: The UUID of the Parent Partner Attachment
        :param pulumi.Input[_builtins.str] redundancy_zone: The redundancy zone for the NaaS
        :param pulumi.Input[_builtins.str] region: The region where the Partner Attachment will be created
        :param pulumi.Input[_builtins.str] state: The state of the Partner Attachment
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] vpc_ids: The list of VPC IDs to attach the Partner Attachment to
        """
        if bgp is not None:
            pulumi.set(__self__, "bgp", bgp)
        if childrens is not None:
            pulumi.set(__self__, "childrens", childrens)
        if connection_bandwidth_in_mbps is not None:
            pulumi.set(__self__, "connection_bandwidth_in_mbps", connection_bandwidth_in_mbps)
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if naas_provider is not None:
            pulumi.set(__self__, "naas_provider", naas_provider)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if parent_uuid is not None:
            pulumi.set(__self__, "parent_uuid", parent_uuid)
        if redundancy_zone is not None:
            pulumi.set(__self__, "redundancy_zone", redundancy_zone)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if vpc_ids is not None:
            pulumi.set(__self__, "vpc_ids", vpc_ids)

    @_builtins.property
    @pulumi.getter
    def bgp(self) -> Optional[pulumi.Input['PartnerAttachmentBgpArgs']]:
        return pulumi.get(self, "bgp")

    @bgp.setter
    def bgp(self, value: Optional[pulumi.Input['PartnerAttachmentBgpArgs']]):
        pulumi.set(self, "bgp", value)

    @_builtins.property
    @pulumi.getter
    def childrens(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The children uuids of Partner Attachment
        """
        return pulumi.get(self, "childrens")

    @childrens.setter
    def childrens(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "childrens", value)

    @_builtins.property
    @pulumi.getter(name="connectionBandwidthInMbps")
    def connection_bandwidth_in_mbps(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The connection bandwidth in Mbps
        """
        return pulumi.get(self, "connection_bandwidth_in_mbps")

    @connection_bandwidth_in_mbps.setter
    def connection_bandwidth_in_mbps(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "connection_bandwidth_in_mbps", value)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The date and time when the Partner Attachment was created
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "created_at", value)

    @_builtins.property
    @pulumi.getter(name="naasProvider")
    def naas_provider(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The NaaS provider
        """
        return pulumi.get(self, "naas_provider")

    @naas_provider.setter
    def naas_provider(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "naas_provider", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the Partner Attachment
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="parentUuid")
    def parent_uuid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of the Parent Partner Attachment
        """
        return pulumi.get(self, "parent_uuid")

    @parent_uuid.setter
    def parent_uuid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "parent_uuid", value)

    @_builtins.property
    @pulumi.getter(name="redundancyZone")
    def redundancy_zone(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The redundancy zone for the NaaS
        """
        return pulumi.get(self, "redundancy_zone")

    @redundancy_zone.setter
    def redundancy_zone(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "redundancy_zone", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region where the Partner Attachment will be created
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The state of the Partner Attachment
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "state", value)

    @_builtins.property
    @pulumi.getter(name="vpcIds")
    def vpc_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The list of VPC IDs to attach the Partner Attachment to
        """
        return pulumi.get(self, "vpc_ids")

    @vpc_ids.setter
    def vpc_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "vpc_ids", value)


@pulumi.type_token("digitalocean:index/partnerAttachment:PartnerAttachment")
class PartnerAttachment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 bgp: Optional[pulumi.Input[Union['PartnerAttachmentBgpArgs', 'PartnerAttachmentBgpArgsDict']]] = None,
                 connection_bandwidth_in_mbps: Optional[pulumi.Input[_builtins.int]] = None,
                 naas_provider: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 parent_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 redundancy_zone: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        """
        Create a PartnerAttachment resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.int] connection_bandwidth_in_mbps: The connection bandwidth in Mbps
        :param pulumi.Input[_builtins.str] naas_provider: The NaaS provider
        :param pulumi.Input[_builtins.str] name: The name of the Partner Attachment
        :param pulumi.Input[_builtins.str] parent_uuid: The UUID of the Parent Partner Attachment
        :param pulumi.Input[_builtins.str] redundancy_zone: The redundancy zone for the NaaS
        :param pulumi.Input[_builtins.str] region: The region where the Partner Attachment will be created
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] vpc_ids: The list of VPC IDs to attach the Partner Attachment to
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PartnerAttachmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a PartnerAttachment resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param PartnerAttachmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PartnerAttachmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 bgp: Optional[pulumi.Input[Union['PartnerAttachmentBgpArgs', 'PartnerAttachmentBgpArgsDict']]] = None,
                 connection_bandwidth_in_mbps: Optional[pulumi.Input[_builtins.int]] = None,
                 naas_provider: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 parent_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 redundancy_zone: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PartnerAttachmentArgs.__new__(PartnerAttachmentArgs)

            __props__.__dict__["bgp"] = bgp
            if connection_bandwidth_in_mbps is None and not opts.urn:
                raise TypeError("Missing required property 'connection_bandwidth_in_mbps'")
            __props__.__dict__["connection_bandwidth_in_mbps"] = connection_bandwidth_in_mbps
            if naas_provider is None and not opts.urn:
                raise TypeError("Missing required property 'naas_provider'")
            __props__.__dict__["naas_provider"] = naas_provider
            __props__.__dict__["name"] = name
            __props__.__dict__["parent_uuid"] = parent_uuid
            __props__.__dict__["redundancy_zone"] = redundancy_zone
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
            if vpc_ids is None and not opts.urn:
                raise TypeError("Missing required property 'vpc_ids'")
            __props__.__dict__["vpc_ids"] = vpc_ids
            __props__.__dict__["childrens"] = None
            __props__.__dict__["created_at"] = None
            __props__.__dict__["state"] = None
        super(PartnerAttachment, __self__).__init__(
            'digitalocean:index/partnerAttachment:PartnerAttachment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            bgp: Optional[pulumi.Input[Union['PartnerAttachmentBgpArgs', 'PartnerAttachmentBgpArgsDict']]] = None,
            childrens: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            connection_bandwidth_in_mbps: Optional[pulumi.Input[_builtins.int]] = None,
            created_at: Optional[pulumi.Input[_builtins.str]] = None,
            naas_provider: Optional[pulumi.Input[_builtins.str]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            parent_uuid: Optional[pulumi.Input[_builtins.str]] = None,
            redundancy_zone: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            state: Optional[pulumi.Input[_builtins.str]] = None,
            vpc_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None) -> 'PartnerAttachment':
        """
        Get an existing PartnerAttachment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] childrens: The children uuids of Partner Attachment
        :param pulumi.Input[_builtins.int] connection_bandwidth_in_mbps: The connection bandwidth in Mbps
        :param pulumi.Input[_builtins.str] created_at: The date and time when the Partner Attachment was created
        :param pulumi.Input[_builtins.str] naas_provider: The NaaS provider
        :param pulumi.Input[_builtins.str] name: The name of the Partner Attachment
        :param pulumi.Input[_builtins.str] parent_uuid: The UUID of the Parent Partner Attachment
        :param pulumi.Input[_builtins.str] redundancy_zone: The redundancy zone for the NaaS
        :param pulumi.Input[_builtins.str] region: The region where the Partner Attachment will be created
        :param pulumi.Input[_builtins.str] state: The state of the Partner Attachment
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] vpc_ids: The list of VPC IDs to attach the Partner Attachment to
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PartnerAttachmentState.__new__(_PartnerAttachmentState)

        __props__.__dict__["bgp"] = bgp
        __props__.__dict__["childrens"] = childrens
        __props__.__dict__["connection_bandwidth_in_mbps"] = connection_bandwidth_in_mbps
        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["naas_provider"] = naas_provider
        __props__.__dict__["name"] = name
        __props__.__dict__["parent_uuid"] = parent_uuid
        __props__.__dict__["redundancy_zone"] = redundancy_zone
        __props__.__dict__["region"] = region
        __props__.__dict__["state"] = state
        __props__.__dict__["vpc_ids"] = vpc_ids
        return PartnerAttachment(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def bgp(self) -> pulumi.Output['outputs.PartnerAttachmentBgp']:
        return pulumi.get(self, "bgp")

    @_builtins.property
    @pulumi.getter
    def childrens(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        The children uuids of Partner Attachment
        """
        return pulumi.get(self, "childrens")

    @_builtins.property
    @pulumi.getter(name="connectionBandwidthInMbps")
    def connection_bandwidth_in_mbps(self) -> pulumi.Output[_builtins.int]:
        """
        The connection bandwidth in Mbps
        """
        return pulumi.get(self, "connection_bandwidth_in_mbps")

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[_builtins.str]:
        """
        The date and time when the Partner Attachment was created
        """
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="naasProvider")
    def naas_provider(self) -> pulumi.Output[_builtins.str]:
        """
        The NaaS provider
        """
        return pulumi.get(self, "naas_provider")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the Partner Attachment
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="parentUuid")
    def parent_uuid(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The UUID of the Parent Partner Attachment
        """
        return pulumi.get(self, "parent_uuid")

    @_builtins.property
    @pulumi.getter(name="redundancyZone")
    def redundancy_zone(self) -> pulumi.Output[_builtins.str]:
        """
        The redundancy zone for the NaaS
        """
        return pulumi.get(self, "redundancy_zone")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region where the Partner Attachment will be created
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def state(self) -> pulumi.Output[_builtins.str]:
        """
        The state of the Partner Attachment
        """
        return pulumi.get(self, "state")

    @_builtins.property
    @pulumi.getter(name="vpcIds")
    def vpc_ids(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        The list of VPC IDs to attach the Partner Attachment to
        """
        return pulumi.get(self, "vpc_ids")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetFirewallResult',
    'AwaitableGetFirewallResult',
    'get_firewall',
    'get_firewall_output',
]

@pulumi.output_type
class GetFirewallResult:
    """
    A collection of values returned by getFirewall.
    """
    def __init__(__self__, created_at=None, droplet_ids=None, firewall_id=None, id=None, inbound_rules=None, name=None, outbound_rules=None, pending_changes=None, status=None, tags=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if droplet_ids and not isinstance(droplet_ids, list):
            raise TypeError("Expected argument 'droplet_ids' to be a list")
        pulumi.set(__self__, "droplet_ids", droplet_ids)
        if firewall_id and not isinstance(firewall_id, str):
            raise TypeError("Expected argument 'firewall_id' to be a str")
        pulumi.set(__self__, "firewall_id", firewall_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if inbound_rules and not isinstance(inbound_rules, list):
            raise TypeError("Expected argument 'inbound_rules' to be a list")
        pulumi.set(__self__, "inbound_rules", inbound_rules)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if outbound_rules and not isinstance(outbound_rules, list):
            raise TypeError("Expected argument 'outbound_rules' to be a list")
        pulumi.set(__self__, "outbound_rules", outbound_rules)
        if pending_changes and not isinstance(pending_changes, list):
            raise TypeError("Expected argument 'pending_changes' to be a list")
        pulumi.set(__self__, "pending_changes", pending_changes)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> _builtins.str:
        """
        A time value given in ISO8601 combined date and time format
        that represents when the Firewall was created.
        """
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="dropletIds")
    def droplet_ids(self) -> Sequence[_builtins.int]:
        """
        The list of the IDs of the Droplets assigned to
        the Firewall.
        """
        return pulumi.get(self, "droplet_ids")

    @_builtins.property
    @pulumi.getter(name="firewallId")
    def firewall_id(self) -> _builtins.str:
        return pulumi.get(self, "firewall_id")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="inboundRules")
    def inbound_rules(self) -> Sequence['outputs.GetFirewallInboundRuleResult']:
        return pulumi.get(self, "inbound_rules")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        The name of the Firewall.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="outboundRules")
    def outbound_rules(self) -> Sequence['outputs.GetFirewallOutboundRuleResult']:
        return pulumi.get(self, "outbound_rules")

    @_builtins.property
    @pulumi.getter(name="pendingChanges")
    def pending_changes(self) -> Sequence['outputs.GetFirewallPendingChangeResult']:
        """
        A set of object containing the fields, `droplet_id`,
        `removing`, and `status`.  It is provided to detail exactly which Droplets
        are having their security policies updated.  When empty, all changes
        have been successfully applied.
        """
        return pulumi.get(self, "pending_changes")

    @_builtins.property
    @pulumi.getter
    def status(self) -> _builtins.str:
        """
        A status string indicating the current state of the Firewall.
        This can be "waiting", "succeeded", or "failed".
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Sequence[_builtins.str]:
        """
        The names of the Tags assigned to the Firewall.
        """
        return pulumi.get(self, "tags")


class AwaitableGetFirewallResult(GetFirewallResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetFirewallResult(
            created_at=self.created_at,
            droplet_ids=self.droplet_ids,
            firewall_id=self.firewall_id,
            id=self.id,
            inbound_rules=self.inbound_rules,
            name=self.name,
            outbound_rules=self.outbound_rules,
            pending_changes=self.pending_changes,
            status=self.status,
            tags=self.tags)


def get_firewall(droplet_ids: Optional[Sequence[_builtins.int]] = None,
                 firewall_id: Optional[_builtins.str] = None,
                 inbound_rules: Optional[Sequence[Union['GetFirewallInboundRuleArgs', 'GetFirewallInboundRuleArgsDict']]] = None,
                 outbound_rules: Optional[Sequence[Union['GetFirewallOutboundRuleArgs', 'GetFirewallOutboundRuleArgsDict']]] = None,
                 tags: Optional[Sequence[_builtins.str]] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetFirewallResult:
    """
    Get information on a DigitalOcean Firewall.

    ## Example Usage

    Get the firewall:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_firewall(firewall_id="1df48973-6eef-4214-854f-fa7726e7e583")
    pulumi.export("exampleFirewallName", example.name)
    ```


    :param Sequence[_builtins.int] droplet_ids: The list of the IDs of the Droplets assigned to
           the Firewall.
    :param _builtins.str firewall_id: The ID of the firewall to retrieve information
           about.
    :param Sequence[_builtins.str] tags: The names of the Tags assigned to the Firewall.
    """
    __args__ = dict()
    __args__['dropletIds'] = droplet_ids
    __args__['firewallId'] = firewall_id
    __args__['inboundRules'] = inbound_rules
    __args__['outboundRules'] = outbound_rules
    __args__['tags'] = tags
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getFirewall:getFirewall', __args__, opts=opts, typ=GetFirewallResult).value

    return AwaitableGetFirewallResult(
        created_at=pulumi.get(__ret__, 'created_at'),
        droplet_ids=pulumi.get(__ret__, 'droplet_ids'),
        firewall_id=pulumi.get(__ret__, 'firewall_id'),
        id=pulumi.get(__ret__, 'id'),
        inbound_rules=pulumi.get(__ret__, 'inbound_rules'),
        name=pulumi.get(__ret__, 'name'),
        outbound_rules=pulumi.get(__ret__, 'outbound_rules'),
        pending_changes=pulumi.get(__ret__, 'pending_changes'),
        status=pulumi.get(__ret__, 'status'),
        tags=pulumi.get(__ret__, 'tags'))
def get_firewall_output(droplet_ids: Optional[pulumi.Input[Optional[Sequence[_builtins.int]]]] = None,
                        firewall_id: Optional[pulumi.Input[_builtins.str]] = None,
                        inbound_rules: Optional[pulumi.Input[Optional[Sequence[Union['GetFirewallInboundRuleArgs', 'GetFirewallInboundRuleArgsDict']]]]] = None,
                        outbound_rules: Optional[pulumi.Input[Optional[Sequence[Union['GetFirewallOutboundRuleArgs', 'GetFirewallOutboundRuleArgsDict']]]]] = None,
                        tags: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                        opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetFirewallResult]:
    """
    Get information on a DigitalOcean Firewall.

    ## Example Usage

    Get the firewall:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_firewall(firewall_id="1df48973-6eef-4214-854f-fa7726e7e583")
    pulumi.export("exampleFirewallName", example.name)
    ```


    :param Sequence[_builtins.int] droplet_ids: The list of the IDs of the Droplets assigned to
           the Firewall.
    :param _builtins.str firewall_id: The ID of the firewall to retrieve information
           about.
    :param Sequence[_builtins.str] tags: The names of the Tags assigned to the Firewall.
    """
    __args__ = dict()
    __args__['dropletIds'] = droplet_ids
    __args__['firewallId'] = firewall_id
    __args__['inboundRules'] = inbound_rules
    __args__['outboundRules'] = outbound_rules
    __args__['tags'] = tags
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getFirewall:getFirewall', __args__, opts=opts, typ=GetFirewallResult)
    return __ret__.apply(lambda __response__: GetFirewallResult(
        created_at=pulumi.get(__response__, 'created_at'),
        droplet_ids=pulumi.get(__response__, 'droplet_ids'),
        firewall_id=pulumi.get(__response__, 'firewall_id'),
        id=pulumi.get(__response__, 'id'),
        inbound_rules=pulumi.get(__response__, 'inbound_rules'),
        name=pulumi.get(__response__, 'name'),
        outbound_rules=pulumi.get(__response__, 'outbound_rules'),
        pending_changes=pulumi.get(__response__, 'pending_changes'),
        status=pulumi.get(__response__, 'status'),
        tags=pulumi.get(__response__, 'tags')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import pulumi
from enum import Enum

__all__ = [
    'Algorithm',
    'CertificateType',
    'DatabaseSlug',
    'DropletSlug',
    'FileSystemType',
    'Protocol',
    'RecordType',
    'Region',
]


@pulumi.type_token("digitalocean:index:Algorithm")
class Algorithm(_builtins.str, Enum):
    ROUND_ROBIN = "round_robin"
    LEAST_CONNECTIONS = "least_connections"


@pulumi.type_token("digitalocean:index:CertificateType")
class CertificateType(_builtins.str, Enum):
    LETS_ENCRYPT = "lets_encrypt"
    CUSTOM = "custom"


@pulumi.type_token("digitalocean:index:DatabaseSlug")
class DatabaseSlug(_builtins.str, Enum):
    D_B_1_VPCU1_GB = "db-s-1vcpu-1gb"
    D_B_1_VPCU2_GB = "db-s-1vcpu-2gb"
    D_B_2_VPCU4_GB = "db-s-2vcpu-4gb"
    D_B_4_VPCU8_GB = "db-s-4vcpu-8gb"
    D_B_6_VPCU16_GB = "db-s-6vcpu-16gb"
    D_B_8_VPCU32_GB = "db-s-8vcpu-32gb"
    D_B_16_VPCU64_GB = "db-s-16vcpu-64gb"


@pulumi.type_token("digitalocean:index:DropletSlug")
class DropletSlug(_builtins.str, Enum):
    DROPLET_S1_VCPU512_MB10_GB = "s-1vcpu-512mb-10gb"
    DROPLET_S1_VCPU1_GB = "s-1vcpu-1gb"
    DROPLET_S1_VCPU1_G_B_AMD = "s-1vcpu-1gb-amd"
    DROPLET_S1_VCPU1_G_B_INTEL = "s-1vcpu-1gb-intel"
    DROPLET_S1_VCPU1_GB35_G_B_INTEL = "s-1vcpu-1gb-35gb-intel"
    DROPLET_S1_VCPU2_GB = "s-1vcpu-2gb"
    DROPLET_S1_VCPU2_G_B_AMD = "s-1vcpu-2gb-amd"
    DROPLET_S1_VCPU2_G_B_INTEL = "s-1vcpu-2gb-intel"
    DROPLET_S1_VCPU2_GB70_G_B_INTEL = "s-1vcpu-2gb-70gb-intel"
    DROPLET_S2_VCPU2_GB = "s-2vcpu-2gb"
    DROPLET_S2_VCPU2_G_B_AMD = "s-2vcpu-2gb-amd"
    DROPLET_S2_VCPU2_G_B_INTEL = "s-2vcpu-2gb-intel"
    DROPLET_S2_VCPU2_GB90_G_B_INTEL = "s-2vcpu-2gb-90gb-intel"
    DROPLET_S2_VCPU4_GB = "s-2vcpu-4gb"
    DROPLET_S2_VCPU4_G_B_AMD = "s-2vcpu-4gb-amd"
    DROPLET_S2_VCPU4_G_B_INTEL = "s-2vcpu-4gb-intel"
    DROPLET_S2_VCPU4_GB120_G_B_INTEL = "s-2vcpu-4gb-120gb-intel"
    DROPLET_S2_VCPU8_G_B_AMD = "s-2vcpu-8gb-amd"
    DROPLET_C2 = "c-2"
    DROPLET_C22_VCPU4_GB = "c2-2vcpu-4gb"
    DROPLET_S2_VCPU8_GB160_G_B_INTEL = "s-2vcpu-8gb-160gb-intel"
    DROPLET_S4_VCPU8_GB = "s-4vcpu-8gb"
    DROPLET_S4_VCPU8_G_B_AMD = "s-4vcpu-8gb-amd"
    DROPLET_S4_VCPU8_G_B_INTEL = "s-4vcpu-8gb-intel"
    DROPLET_G2_VCPU8_GB = "g-2vcpu-8gb"
    DROPLET_S4_VCPU8_GB240_G_B_INTEL = "s-4vcpu-8gb-240gb-intel"
    DROPLET_GD2_VCPU8_GB = "gd-2vcpu-8gb"
    DROPLET_G2_VCPU8_G_B_INTEL = "g-2vcpu-8gb-intel"
    DROPLET_GD2_VCPU8_G_B_INTEL = "gd-2vcpu-8gb-intel"
    DROPLET_S4_VCPU16_G_B_AMD = "s-4vcpu-16gb-amd"
    DROPLET_M2_VCPU16_GB = "m-2vcpu-16gb"
    DROPLET_C4 = "c-4"
    DROPLET_C24_VCPU8_GB = "c2-4vcpu-8gb"
    DROPLET_S4_VCPU16_GB320_G_B_INTEL = "s-4vcpu-16gb-320gb-intel"
    DROPLET_S8_VCPU16_GB = "s-8vcpu-16gb"
    DROPLET_M2_VCPU16_G_B_INTEL = "m-2vcpu-16gb-intel"
    DROPLET_M32_VCPU16_GB = "m3-2vcpu-16gb"
    DROPLET_C4_INTEL = "c-4-intel"
    DROPLET_M32_VCPU16_G_B_INTEL = "m3-2vcpu-16gb-intel"
    DROPLET_S8_VCPU16_G_B_AMD = "s-8vcpu-16gb-amd"
    DROPLET_S8_VCPU16_G_B_INTEL = "s-8vcpu-16gb-intel"
    DROPLET_C24_VCPU8_G_B_INTEL = "c2-4vcpu-8gb-intel"
    DROPLET_G4_VCPU16_GB = "g-4vcpu-16gb"
    DROPLET_S8_VCPU16_GB480_G_B_INTEL = "s-8vcpu-16gb-480gb-intel"
    DROPLET_SO2_VCPU16_G_B_INTEL = "so-2vcpu-16gb-intel"
    DROPLET_SO2_VCPU16_GB = "so-2vcpu-16gb"
    DROPLET_M62_VCPU16_GB = "m6-2vcpu-16gb"
    DROPLET_GD4_VCPU16_GB = "gd-4vcpu-16gb"
    DROPLET_SO1_52_VCPU16_G_B_INTEL = "so1_5-2vcpu-16gb-intel"
    DROPLET_G4_VCPU16_G_B_INTEL = "g-4vcpu-16gb-intel"
    DROPLET_GD4_VCPU16_G_B_INTEL = "gd-4vcpu-16gb-intel"
    DROPLET_SO1_52_VCPU16_GB = "so1_5-2vcpu-16gb"
    DROPLET_S8_VCPU32_G_B_AMD = "s-8vcpu-32gb-amd"
    DROPLET_M4_VCPU32_GB = "m-4vcpu-32gb"
    DROPLET_C8 = "c-8"
    DROPLET_C28_VCPU16_GB = "c2-8vcpu-16gb"
    DROPLET_S8_VCPU32_GB640_G_B_INTEL = "s-8vcpu-32gb-640gb-intel"
    DROPLET_M4_VCPU32_G_B_INTEL = "m-4vcpu-32gb-intel"
    DROPLET_M34_VCPU32_GB = "m3-4vcpu-32gb"
    DROPLET_C8_INTEL = "c-8-intel"
    DROPLET_M34_VCPU32_G_B_INTEL = "m3-4vcpu-32gb-intel"
    DROPLET_C28_VCPU16_G_B_INTEL = "c2-8vcpu-16gb-intel"
    DROPLET_G8_VCPU32_GB = "g-8vcpu-32gb"
    DROPLET_SO4_VCPU32_G_B_INTEL = "so-4vcpu-32gb-intel"
    DROPLET_SO4_VCPU32_GB = "so-4vcpu-32gb"
    DROPLET_M64_VCPU32_GB = "m6-4vcpu-32gb"
    DROPLET_GD8_VCPU32_GB = "gd-8vcpu-32gb"
    DROPLET_SO1_54_VCPU32_G_B_INTEL = "so1_5-4vcpu-32gb-intel"
    DROPLET_G8_VCPU32_G_B_INTEL = "g-8vcpu-32gb-intel"
    DROPLET_GD8_VCPU32_G_B_INTEL = "gd-8vcpu-32gb-intel"
    DROPLET_SO1_54_VCPU32_GB = "so1_5-4vcpu-32gb"
    DROPLET_M8_VCPU64_GB = "m-8vcpu-64gb"
    DROPLET_C16 = "c-16"
    DROPLET_C216_VCPU32_GB = "c2-16vcpu-32gb"
    DROPLET_M8_VCPU64_G_B_INTEL = "m-8vcpu-64gb-intel"
    DROPLET_M38_VCPU64_GB = "m3-8vcpu-64gb"
    DROPLET_C16_INTEL = "c-16-intel"
    DROPLET_M38_VCPU64_G_B_INTEL = "m3-8vcpu-64gb-intel"
    DROPLET_C216_VCPU32_G_B_INTEL = "c2-16vcpu-32gb-intel"
    DROPLET_G16_VCPU64_GB = "g-16vcpu-64gb"
    DROPLET_SO8_VCPU64_G_B_INTEL = "so-8vcpu-64gb-intel"
    DROPLET_SO8_VCPU64_GB = "so-8vcpu-64gb"
    DROPLET_M68_VCPU64_GB = "m6-8vcpu-64gb"
    DROPLET_GD16_VCPU64_GB = "gd-16vcpu-64gb"
    DROPLET_SO1_58_VCPU64_G_B_INTEL = "so1_5-8vcpu-64gb-intel"
    DROPLET_G16_VCPU64_G_B_INTEL = "g-16vcpu-64gb-intel"
    DROPLET_GD16_VCPU64_G_B_INTEL = "gd-16vcpu-64gb-intel"
    DROPLET_SO1_58_VCPU64_GB = "so1_5-8vcpu-64gb"
    DROPLET_M16_VCPU128_GB = "m-16vcpu-128gb"
    DROPLET_C32 = "c-32"
    DROPLET_C232_VCPU64_GB = "c2-32vcpu-64gb"
    DROPLET_M16_VCPU128_G_B_INTEL = "m-16vcpu-128gb-intel"
    DROPLET_M316_VCPU128_GB = "m3-16vcpu-128gb"
    DROPLET_C32_INTEL = "c-32-intel"
    DROPLET_M316_VCPU128_G_B_INTEL = "m3-16vcpu-128gb-intel"
    DROPLET_C232_VCPU64_G_B_INTEL = "c2-32vcpu-64gb-intel"
    DROPLET_C48 = "c-48"
    DROPLET_M24_VCPU192_GB = "m-24vcpu-192gb"
    DROPLET_G32_VCPU128_GB = "g-32vcpu-128gb"
    DROPLET_SO16_VCPU128_G_B_INTEL = "so-16vcpu-128gb-intel"
    DROPLET_SO16_VCPU128_GB = "so-16vcpu-128gb"
    DROPLET_M616_VCPU128_GB = "m6-16vcpu-128gb"
    DROPLET_GD32_VCPU128_GB = "gd-32vcpu-128gb"
    DROPLET_SO1_516_VCPU128_G_B_INTEL = "so1_5-16vcpu-128gb-intel"
    DROPLET_C248_VCPU96_GB = "c2-48vcpu-96gb"
    DROPLET_M24_VCPU192_G_B_INTEL = "m-24vcpu-192gb-intel"
    DROPLET_G32_VCPU128_G_B_INTEL = "g-32vcpu-128gb-intel"
    DROPLET_M324_VCPU192_GB = "m3-24vcpu-192gb"
    DROPLET_G40_VCPU160_GB = "g-40vcpu-160gb"
    DROPLET_GD32_VCPU128_G_B_INTEL = "gd-32vcpu-128gb-intel"
    DROPLET_SO1_516_VCPU128_GB = "so1_5-16vcpu-128gb"
    DROPLET_C48_INTEL = "c-48-intel"
    DROPLET_M324_VCPU192_G_B_INTEL = "m3-24vcpu-192gb-intel"
    DROPLET_M32_VCPU256_GB = "m-32vcpu-256gb"
    DROPLET_GD40_VCPU160_GB = "gd-40vcpu-160gb"
    DROPLET_C248_VCPU96_G_B_INTEL = "c2-48vcpu-96gb-intel"
    DROPLET_SO24_VCPU192_G_B_INTEL = "so-24vcpu-192gb-intel"
    DROPLET_SO24_VCPU192_GB = "so-24vcpu-192gb"
    DROPLET_M624_VCPU192_GB = "m6-24vcpu-192gb"
    DROPLET_M32_VCPU256_G_B_INTEL = "m-32vcpu-256gb-intel"
    DROPLET_C60_INTEL = "c-60-intel"
    DROPLET_M332_VCPU256_GB = "m3-32vcpu-256gb"
    DROPLET_SO1_524_VCPU192_G_B_INTEL = "so1_5-24vcpu-192gb-intel"
    DROPLET_M332_VCPU256_G_B_INTEL = "m3-32vcpu-256gb-intel"
    DROPLET_G48_VCPU192_G_B_INTEL = "g-48vcpu-192gb-intel"
    DROPLET_C260_VCPU120_G_B_INTEL = "c2-60vcpu-120gb-intel"
    DROPLET_GD48_VCPU192_G_B_INTEL = "gd-48vcpu-192gb-intel"
    DROPLET_SO1_524_VCPU192_GB = "so1_5-24vcpu-192gb"
    DROPLET_SO32_VCPU256_G_B_INTEL = "so-32vcpu-256gb-intel"
    DROPLET_SO32_VCPU256_GB = "so-32vcpu-256gb"
    DROPLET_M632_VCPU256_GB = "m6-32vcpu-256gb"
    DROPLET_SO1_532_VCPU256_G_B_INTEL = "so1_5-32vcpu-256gb-intel"
    DROPLET_G60_VCPU240_G_B_INTEL = "g-60vcpu-240gb-intel"
    DROPLET_M48_VCPU384_G_B_INTEL = "m-48vcpu-384gb-intel"
    DROPLET_GD60_VCPU240_G_B_INTEL = "gd-60vcpu-240gb-intel"
    DROPLET_GPUH100_X180_GB = "gpu-h100x1-80gb"
    DROPLET_SO1_532_VCPU256_GB = "so1_5-32vcpu-256gb"
    DROPLET_M348_VCPU384_G_B_INTEL = "m3-48vcpu-384gb-intel"
    DROPLET_SO48_VCPU384_G_B_INTEL = "so-48vcpu-384gb-intel"
    DROPLET_GPUH100_X180_GB200 = "gpu-h100x1-80gb-200"
    DROPLET_GPUH100_X8640_GB = "gpu-h100x8-640gb"
    DROPLET_GPUH100_X8640_GB200 = "gpu-h100x8-640gb-200"
    DROPLET16_GB = "16gb"
    DROPLET1_GB = "1gb"
    DROPLET2_GB = "2gb"
    DROPLET32_GB = "32gb"
    DROPLET48_GB = "48gb"
    DROPLET4_GB = "4gb"
    DROPLET512MB = "512mb"
    DROPLET64_GB = "64gb"
    DROPLET8_GB = "8gb"
    DROPLET_C22_VCPU8_GB = "c2-4vcpu-8gb"
    DROPLET_S12_VCPU48_GB = "s-12vcpu-48gb"
    DROPLET_S16_VCPU64_GB = "s-16vcpu-64gb"
    DROPLET_S1_VCPU3_GB = "s-1vcpu-3gb"
    DROPLET_S20_VCPU96_GB = "s-20vcpu-96gb"
    DROPLET_S24_VCPU128_GB = "s-24vcpu-128gb"
    DROPLET_S32_VCPU192_GB = "s-32vcpu-192gb"
    DROPLET_S3_VCPU1_GB = "s-3vcpu-1gb"
    DROPLET_S6_VCPU16_GB = "s-6vcpu-16gb"
    DROPLET_S8_VCPU32_GB = "s-8vcpu-32gb"
    DROPLET_SO1516_VCPU128_GB = "so1_5-16vcpu-128gb"
    DROPLET_SO1524_VCPU192_GB = "so1_5-24vcpu-192gb"
    DROPLET_SO152_VCPU16_GB = "so1_5-2vcpu-16gb"
    DROPLET_SO1532_VCPU256_GB = "so1_5-32vcpu-256gb"
    DROPLET_SO154_VCPU32_GB = "so1_5-4vcpu-32gb"
    DROPLET_SO158_VCPU64_GB = "so1_5-8vcpu-64gb"


@pulumi.type_token("digitalocean:index:FileSystemType")
class FileSystemType(_builtins.str, Enum):
    EXT4 = "ext4"
    XFS = "xfs"


@pulumi.type_token("digitalocean:index:Protocol")
class Protocol(_builtins.str, Enum):
    TCP = "tcp"
    UDP = "udp"
    ICMP = "icmp"
    HTTP = "http"
    HTTPS = "https"


@pulumi.type_token("digitalocean:index:RecordType")
class RecordType(_builtins.str, Enum):
    A = "A"
    AAAA = "AAAA"
    CAA = "CAA"
    CNAME = "CNAME"
    MX = "MX"
    NS = "NS"
    TXT = "TXT"
    SRV = "SRV"


@pulumi.type_token("digitalocean:index:Region")
class Region(_builtins.str, Enum):
    NYC1 = "nyc1"
    NYC2 = "nyc2"
    NYC3 = "nyc3"
    SGP1 = "sgp1"
    LON1 = "lon1"
    AMS2 = "ams2"
    AMS3 = "ams3"
    FRA1 = "fra1"
    TOR1 = "tor1"
    SFO1 = "sfo1"
    SFO2 = "sfo2"
    SFO3 = "sfo3"
    BLR1 = "blr1"
    SYD1 = "syd1"



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetLoadBalancerResult',
    'AwaitableGetLoadBalancerResult',
    'get_load_balancer',
    'get_load_balancer_output',
]

@pulumi.output_type
class GetLoadBalancerResult:
    """
    A collection of values returned by getLoadBalancer.
    """
    def __init__(__self__, algorithm=None, disable_lets_encrypt_dns_records=None, domains=None, droplet_ids=None, droplet_tag=None, enable_backend_keepalive=None, enable_proxy_protocol=None, firewalls=None, forwarding_rules=None, glb_settings=None, healthchecks=None, http_idle_timeout_seconds=None, id=None, ip=None, ipv6=None, load_balancer_urn=None, name=None, network=None, project_id=None, redirect_http_to_https=None, region=None, size=None, size_unit=None, status=None, sticky_sessions=None, target_load_balancer_ids=None, type=None, vpc_uuid=None):
        if algorithm and not isinstance(algorithm, str):
            raise TypeError("Expected argument 'algorithm' to be a str")
        pulumi.set(__self__, "algorithm", algorithm)
        if disable_lets_encrypt_dns_records and not isinstance(disable_lets_encrypt_dns_records, bool):
            raise TypeError("Expected argument 'disable_lets_encrypt_dns_records' to be a bool")
        pulumi.set(__self__, "disable_lets_encrypt_dns_records", disable_lets_encrypt_dns_records)
        if domains and not isinstance(domains, list):
            raise TypeError("Expected argument 'domains' to be a list")
        pulumi.set(__self__, "domains", domains)
        if droplet_ids and not isinstance(droplet_ids, list):
            raise TypeError("Expected argument 'droplet_ids' to be a list")
        pulumi.set(__self__, "droplet_ids", droplet_ids)
        if droplet_tag and not isinstance(droplet_tag, str):
            raise TypeError("Expected argument 'droplet_tag' to be a str")
        pulumi.set(__self__, "droplet_tag", droplet_tag)
        if enable_backend_keepalive and not isinstance(enable_backend_keepalive, bool):
            raise TypeError("Expected argument 'enable_backend_keepalive' to be a bool")
        pulumi.set(__self__, "enable_backend_keepalive", enable_backend_keepalive)
        if enable_proxy_protocol and not isinstance(enable_proxy_protocol, bool):
            raise TypeError("Expected argument 'enable_proxy_protocol' to be a bool")
        pulumi.set(__self__, "enable_proxy_protocol", enable_proxy_protocol)
        if firewalls and not isinstance(firewalls, list):
            raise TypeError("Expected argument 'firewalls' to be a list")
        pulumi.set(__self__, "firewalls", firewalls)
        if forwarding_rules and not isinstance(forwarding_rules, list):
            raise TypeError("Expected argument 'forwarding_rules' to be a list")
        pulumi.set(__self__, "forwarding_rules", forwarding_rules)
        if glb_settings and not isinstance(glb_settings, list):
            raise TypeError("Expected argument 'glb_settings' to be a list")
        pulumi.set(__self__, "glb_settings", glb_settings)
        if healthchecks and not isinstance(healthchecks, list):
            raise TypeError("Expected argument 'healthchecks' to be a list")
        pulumi.set(__self__, "healthchecks", healthchecks)
        if http_idle_timeout_seconds and not isinstance(http_idle_timeout_seconds, int):
            raise TypeError("Expected argument 'http_idle_timeout_seconds' to be a int")
        pulumi.set(__self__, "http_idle_timeout_seconds", http_idle_timeout_seconds)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip and not isinstance(ip, str):
            raise TypeError("Expected argument 'ip' to be a str")
        pulumi.set(__self__, "ip", ip)
        if ipv6 and not isinstance(ipv6, str):
            raise TypeError("Expected argument 'ipv6' to be a str")
        pulumi.set(__self__, "ipv6", ipv6)
        if load_balancer_urn and not isinstance(load_balancer_urn, str):
            raise TypeError("Expected argument 'load_balancer_urn' to be a str")
        pulumi.set(__self__, "load_balancer_urn", load_balancer_urn)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if network and not isinstance(network, str):
            raise TypeError("Expected argument 'network' to be a str")
        pulumi.set(__self__, "network", network)
        if project_id and not isinstance(project_id, str):
            raise TypeError("Expected argument 'project_id' to be a str")
        pulumi.set(__self__, "project_id", project_id)
        if redirect_http_to_https and not isinstance(redirect_http_to_https, bool):
            raise TypeError("Expected argument 'redirect_http_to_https' to be a bool")
        pulumi.set(__self__, "redirect_http_to_https", redirect_http_to_https)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if size and not isinstance(size, str):
            raise TypeError("Expected argument 'size' to be a str")
        pulumi.set(__self__, "size", size)
        if size_unit and not isinstance(size_unit, int):
            raise TypeError("Expected argument 'size_unit' to be a int")
        pulumi.set(__self__, "size_unit", size_unit)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)
        if sticky_sessions and not isinstance(sticky_sessions, list):
            raise TypeError("Expected argument 'sticky_sessions' to be a list")
        pulumi.set(__self__, "sticky_sessions", sticky_sessions)
        if target_load_balancer_ids and not isinstance(target_load_balancer_ids, list):
            raise TypeError("Expected argument 'target_load_balancer_ids' to be a list")
        pulumi.set(__self__, "target_load_balancer_ids", target_load_balancer_ids)
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        pulumi.set(__self__, "type", type)
        if vpc_uuid and not isinstance(vpc_uuid, str):
            raise TypeError("Expected argument 'vpc_uuid' to be a str")
        pulumi.set(__self__, "vpc_uuid", vpc_uuid)

    @_builtins.property
    @pulumi.getter
    @_utilities.deprecated("""This field has been deprecated. You can no longer specify an algorithm for load balancers.""")
    def algorithm(self) -> _builtins.str:
        return pulumi.get(self, "algorithm")

    @_builtins.property
    @pulumi.getter(name="disableLetsEncryptDnsRecords")
    def disable_lets_encrypt_dns_records(self) -> _builtins.bool:
        return pulumi.get(self, "disable_lets_encrypt_dns_records")

    @_builtins.property
    @pulumi.getter
    def domains(self) -> Sequence['outputs.GetLoadBalancerDomainResult']:
        return pulumi.get(self, "domains")

    @_builtins.property
    @pulumi.getter(name="dropletIds")
    def droplet_ids(self) -> Sequence[_builtins.int]:
        return pulumi.get(self, "droplet_ids")

    @_builtins.property
    @pulumi.getter(name="dropletTag")
    def droplet_tag(self) -> _builtins.str:
        return pulumi.get(self, "droplet_tag")

    @_builtins.property
    @pulumi.getter(name="enableBackendKeepalive")
    def enable_backend_keepalive(self) -> _builtins.bool:
        return pulumi.get(self, "enable_backend_keepalive")

    @_builtins.property
    @pulumi.getter(name="enableProxyProtocol")
    def enable_proxy_protocol(self) -> _builtins.bool:
        return pulumi.get(self, "enable_proxy_protocol")

    @_builtins.property
    @pulumi.getter
    def firewalls(self) -> Sequence['outputs.GetLoadBalancerFirewallResult']:
        return pulumi.get(self, "firewalls")

    @_builtins.property
    @pulumi.getter(name="forwardingRules")
    def forwarding_rules(self) -> Sequence['outputs.GetLoadBalancerForwardingRuleResult']:
        return pulumi.get(self, "forwarding_rules")

    @_builtins.property
    @pulumi.getter(name="glbSettings")
    def glb_settings(self) -> Sequence['outputs.GetLoadBalancerGlbSettingResult']:
        return pulumi.get(self, "glb_settings")

    @_builtins.property
    @pulumi.getter
    def healthchecks(self) -> Sequence['outputs.GetLoadBalancerHealthcheckResult']:
        return pulumi.get(self, "healthchecks")

    @_builtins.property
    @pulumi.getter(name="httpIdleTimeoutSeconds")
    def http_idle_timeout_seconds(self) -> _builtins.int:
        return pulumi.get(self, "http_idle_timeout_seconds")

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def ip(self) -> _builtins.str:
        return pulumi.get(self, "ip")

    @_builtins.property
    @pulumi.getter
    def ipv6(self) -> _builtins.str:
        return pulumi.get(self, "ipv6")

    @_builtins.property
    @pulumi.getter(name="loadBalancerUrn")
    def load_balancer_urn(self) -> _builtins.str:
        return pulumi.get(self, "load_balancer_urn")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def network(self) -> _builtins.str:
        return pulumi.get(self, "network")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> _builtins.str:
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter(name="redirectHttpToHttps")
    def redirect_http_to_https(self) -> _builtins.bool:
        return pulumi.get(self, "redirect_http_to_https")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def size(self) -> _builtins.str:
        return pulumi.get(self, "size")

    @_builtins.property
    @pulumi.getter(name="sizeUnit")
    def size_unit(self) -> _builtins.int:
        return pulumi.get(self, "size_unit")

    @_builtins.property
    @pulumi.getter
    def status(self) -> _builtins.str:
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter(name="stickySessions")
    def sticky_sessions(self) -> Sequence['outputs.GetLoadBalancerStickySessionResult']:
        return pulumi.get(self, "sticky_sessions")

    @_builtins.property
    @pulumi.getter(name="targetLoadBalancerIds")
    def target_load_balancer_ids(self) -> Sequence[_builtins.str]:
        return pulumi.get(self, "target_load_balancer_ids")

    @_builtins.property
    @pulumi.getter
    def type(self) -> _builtins.str:
        return pulumi.get(self, "type")

    @_builtins.property
    @pulumi.getter(name="vpcUuid")
    def vpc_uuid(self) -> _builtins.str:
        return pulumi.get(self, "vpc_uuid")


class AwaitableGetLoadBalancerResult(GetLoadBalancerResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetLoadBalancerResult(
            algorithm=self.algorithm,
            disable_lets_encrypt_dns_records=self.disable_lets_encrypt_dns_records,
            domains=self.domains,
            droplet_ids=self.droplet_ids,
            droplet_tag=self.droplet_tag,
            enable_backend_keepalive=self.enable_backend_keepalive,
            enable_proxy_protocol=self.enable_proxy_protocol,
            firewalls=self.firewalls,
            forwarding_rules=self.forwarding_rules,
            glb_settings=self.glb_settings,
            healthchecks=self.healthchecks,
            http_idle_timeout_seconds=self.http_idle_timeout_seconds,
            id=self.id,
            ip=self.ip,
            ipv6=self.ipv6,
            load_balancer_urn=self.load_balancer_urn,
            name=self.name,
            network=self.network,
            project_id=self.project_id,
            redirect_http_to_https=self.redirect_http_to_https,
            region=self.region,
            size=self.size,
            size_unit=self.size_unit,
            status=self.status,
            sticky_sessions=self.sticky_sessions,
            target_load_balancer_ids=self.target_load_balancer_ids,
            type=self.type,
            vpc_uuid=self.vpc_uuid)


def get_load_balancer(id: Optional[_builtins.str] = None,
                      name: Optional[_builtins.str] = None,
                      opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetLoadBalancerResult:
    """
    Get information on a load balancer for use in other resources. This data source
    provides all of the load balancers properties as configured on your DigitalOcean
    account. This is useful if the load balancer in question is not managed by
    the provider or you need to utilize any of the load balancers data.

    An error is triggered if the provided load balancer name does not exist.

    ## Example Usage

    Get the load balancer by name:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_load_balancer(name="app")
    pulumi.export("lbOutput", example.ip)
    ```

    Get the load balancer by ID:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_load_balancer(id="loadbalancer_id")
    ```


    :param _builtins.str id: The ID of load balancer.
    :param _builtins.str name: The name of load balancer.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['name'] = name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getLoadBalancer:getLoadBalancer', __args__, opts=opts, typ=GetLoadBalancerResult).value

    return AwaitableGetLoadBalancerResult(
        algorithm=pulumi.get(__ret__, 'algorithm'),
        disable_lets_encrypt_dns_records=pulumi.get(__ret__, 'disable_lets_encrypt_dns_records'),
        domains=pulumi.get(__ret__, 'domains'),
        droplet_ids=pulumi.get(__ret__, 'droplet_ids'),
        droplet_tag=pulumi.get(__ret__, 'droplet_tag'),
        enable_backend_keepalive=pulumi.get(__ret__, 'enable_backend_keepalive'),
        enable_proxy_protocol=pulumi.get(__ret__, 'enable_proxy_protocol'),
        firewalls=pulumi.get(__ret__, 'firewalls'),
        forwarding_rules=pulumi.get(__ret__, 'forwarding_rules'),
        glb_settings=pulumi.get(__ret__, 'glb_settings'),
        healthchecks=pulumi.get(__ret__, 'healthchecks'),
        http_idle_timeout_seconds=pulumi.get(__ret__, 'http_idle_timeout_seconds'),
        id=pulumi.get(__ret__, 'id'),
        ip=pulumi.get(__ret__, 'ip'),
        ipv6=pulumi.get(__ret__, 'ipv6'),
        load_balancer_urn=pulumi.get(__ret__, 'load_balancer_urn'),
        name=pulumi.get(__ret__, 'name'),
        network=pulumi.get(__ret__, 'network'),
        project_id=pulumi.get(__ret__, 'project_id'),
        redirect_http_to_https=pulumi.get(__ret__, 'redirect_http_to_https'),
        region=pulumi.get(__ret__, 'region'),
        size=pulumi.get(__ret__, 'size'),
        size_unit=pulumi.get(__ret__, 'size_unit'),
        status=pulumi.get(__ret__, 'status'),
        sticky_sessions=pulumi.get(__ret__, 'sticky_sessions'),
        target_load_balancer_ids=pulumi.get(__ret__, 'target_load_balancer_ids'),
        type=pulumi.get(__ret__, 'type'),
        vpc_uuid=pulumi.get(__ret__, 'vpc_uuid'))
def get_load_balancer_output(id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                             name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                             opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetLoadBalancerResult]:
    """
    Get information on a load balancer for use in other resources. This data source
    provides all of the load balancers properties as configured on your DigitalOcean
    account. This is useful if the load balancer in question is not managed by
    the provider or you need to utilize any of the load balancers data.

    An error is triggered if the provided load balancer name does not exist.

    ## Example Usage

    Get the load balancer by name:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_load_balancer(name="app")
    pulumi.export("lbOutput", example.ip)
    ```

    Get the load balancer by ID:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    example = digitalocean.get_load_balancer(id="loadbalancer_id")
    ```


    :param _builtins.str id: The ID of load balancer.
    :param _builtins.str name: The name of load balancer.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['name'] = name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getLoadBalancer:getLoadBalancer', __args__, opts=opts, typ=GetLoadBalancerResult)
    return __ret__.apply(lambda __response__: GetLoadBalancerResult(
        algorithm=pulumi.get(__response__, 'algorithm'),
        disable_lets_encrypt_dns_records=pulumi.get(__response__, 'disable_lets_encrypt_dns_records'),
        domains=pulumi.get(__response__, 'domains'),
        droplet_ids=pulumi.get(__response__, 'droplet_ids'),
        droplet_tag=pulumi.get(__response__, 'droplet_tag'),
        enable_backend_keepalive=pulumi.get(__response__, 'enable_backend_keepalive'),
        enable_proxy_protocol=pulumi.get(__response__, 'enable_proxy_protocol'),
        firewalls=pulumi.get(__response__, 'firewalls'),
        forwarding_rules=pulumi.get(__response__, 'forwarding_rules'),
        glb_settings=pulumi.get(__response__, 'glb_settings'),
        healthchecks=pulumi.get(__response__, 'healthchecks'),
        http_idle_timeout_seconds=pulumi.get(__response__, 'http_idle_timeout_seconds'),
        id=pulumi.get(__response__, 'id'),
        ip=pulumi.get(__response__, 'ip'),
        ipv6=pulumi.get(__response__, 'ipv6'),
        load_balancer_urn=pulumi.get(__response__, 'load_balancer_urn'),
        name=pulumi.get(__response__, 'name'),
        network=pulumi.get(__response__, 'network'),
        project_id=pulumi.get(__response__, 'project_id'),
        redirect_http_to_https=pulumi.get(__response__, 'redirect_http_to_https'),
        region=pulumi.get(__response__, 'region'),
        size=pulumi.get(__response__, 'size'),
        size_unit=pulumi.get(__response__, 'size_unit'),
        status=pulumi.get(__response__, 'status'),
        sticky_sessions=pulumi.get(__response__, 'sticky_sessions'),
        target_load_balancer_ids=pulumi.get(__response__, 'target_load_balancer_ids'),
        type=pulumi.get(__response__, 'type'),
        vpc_uuid=pulumi.get(__response__, 'vpc_uuid')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from ._enums import *

__all__ = ['VolumeArgs', 'Volume']

@pulumi.input_type
class VolumeArgs:
    def __init__(__self__, *,
                 region: pulumi.Input[Union[_builtins.str, 'Region']],
                 size: pulumi.Input[_builtins.int],
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 filesystem_type: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_type: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 snapshot_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        The set of arguments for constructing a Volume resource.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: The region that the block storage volume will be created in.
        :param pulumi.Input[_builtins.int] size: The size of the block storage volume in GiB. If updated, can only be expanded.
        :param pulumi.Input[_builtins.str] description: A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        :param pulumi.Input[_builtins.str] filesystem_type: Filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] initial_filesystem_label: Initial filesystem label for the block storage volume.
        :param pulumi.Input[Union[_builtins.str, 'FileSystemType']] initial_filesystem_type: Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] name: A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        :param pulumi.Input[_builtins.str] snapshot_id: The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of the tags to be applied to this Volume.
        """
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "size", size)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if filesystem_type is not None:
            warnings.warn("""This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""", DeprecationWarning)
            pulumi.log.warn("""filesystem_type is deprecated: This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""")
        if filesystem_type is not None:
            pulumi.set(__self__, "filesystem_type", filesystem_type)
        if initial_filesystem_label is not None:
            pulumi.set(__self__, "initial_filesystem_label", initial_filesystem_label)
        if initial_filesystem_type is not None:
            pulumi.set(__self__, "initial_filesystem_type", initial_filesystem_type)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if snapshot_id is not None:
            pulumi.set(__self__, "snapshot_id", snapshot_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Input[Union[_builtins.str, 'Region']]:
        """
        The region that the block storage volume will be created in.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[Union[_builtins.str, 'Region']]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def size(self) -> pulumi.Input[_builtins.int]:
        """
        The size of the block storage volume in GiB. If updated, can only be expanded.
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: pulumi.Input[_builtins.int]):
        pulumi.set(self, "size", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="filesystemType")
    @_utilities.deprecated("""This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""")
    def filesystem_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Filesystem type (`xfs` or `ext4`) for the block storage volume.
        """
        return pulumi.get(self, "filesystem_type")

    @filesystem_type.setter
    def filesystem_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "filesystem_type", value)

    @_builtins.property
    @pulumi.getter(name="initialFilesystemLabel")
    def initial_filesystem_label(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Initial filesystem label for the block storage volume.
        """
        return pulumi.get(self, "initial_filesystem_label")

    @initial_filesystem_label.setter
    def initial_filesystem_label(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "initial_filesystem_label", value)

    @_builtins.property
    @pulumi.getter(name="initialFilesystemType")
    def initial_filesystem_type(self) -> Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]]:
        """
        Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        """
        return pulumi.get(self, "initial_filesystem_type")

    @initial_filesystem_type.setter
    def initial_filesystem_type(self, value: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]]):
        pulumi.set(self, "initial_filesystem_type", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="snapshotId")
    def snapshot_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        """
        return pulumi.get(self, "snapshot_id")

    @snapshot_id.setter
    def snapshot_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "snapshot_id", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of the tags to be applied to this Volume.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _VolumeState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 droplet_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
                 filesystem_type: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_type: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
                 size: Optional[pulumi.Input[_builtins.int]] = None,
                 snapshot_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 volume_urn: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering Volume resources.
        :param pulumi.Input[_builtins.str] description: A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] droplet_ids: A list of associated droplet ids.
        :param pulumi.Input[_builtins.str] filesystem_label: Filesystem label for the block storage volume.
        :param pulumi.Input[_builtins.str] filesystem_type: Filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] initial_filesystem_label: Initial filesystem label for the block storage volume.
        :param pulumi.Input[Union[_builtins.str, 'FileSystemType']] initial_filesystem_type: Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] name: A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: The region that the block storage volume will be created in.
        :param pulumi.Input[_builtins.int] size: The size of the block storage volume in GiB. If updated, can only be expanded.
        :param pulumi.Input[_builtins.str] snapshot_id: The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of the tags to be applied to this Volume.
        :param pulumi.Input[_builtins.str] volume_urn: The uniform resource name for the volume.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if droplet_ids is not None:
            pulumi.set(__self__, "droplet_ids", droplet_ids)
        if filesystem_label is not None:
            pulumi.set(__self__, "filesystem_label", filesystem_label)
        if filesystem_type is not None:
            warnings.warn("""This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""", DeprecationWarning)
            pulumi.log.warn("""filesystem_type is deprecated: This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""")
        if filesystem_type is not None:
            pulumi.set(__self__, "filesystem_type", filesystem_type)
        if initial_filesystem_label is not None:
            pulumi.set(__self__, "initial_filesystem_label", initial_filesystem_label)
        if initial_filesystem_type is not None:
            pulumi.set(__self__, "initial_filesystem_type", initial_filesystem_type)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if size is not None:
            pulumi.set(__self__, "size", size)
        if snapshot_id is not None:
            pulumi.set(__self__, "snapshot_id", snapshot_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if volume_urn is not None:
            pulumi.set(__self__, "volume_urn", volume_urn)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="dropletIds")
    def droplet_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of associated droplet ids.
        """
        return pulumi.get(self, "droplet_ids")

    @droplet_ids.setter
    def droplet_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "droplet_ids", value)

    @_builtins.property
    @pulumi.getter(name="filesystemLabel")
    def filesystem_label(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Filesystem label for the block storage volume.
        """
        return pulumi.get(self, "filesystem_label")

    @filesystem_label.setter
    def filesystem_label(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "filesystem_label", value)

    @_builtins.property
    @pulumi.getter(name="filesystemType")
    @_utilities.deprecated("""This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""")
    def filesystem_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Filesystem type (`xfs` or `ext4`) for the block storage volume.
        """
        return pulumi.get(self, "filesystem_type")

    @filesystem_type.setter
    def filesystem_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "filesystem_type", value)

    @_builtins.property
    @pulumi.getter(name="initialFilesystemLabel")
    def initial_filesystem_label(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Initial filesystem label for the block storage volume.
        """
        return pulumi.get(self, "initial_filesystem_label")

    @initial_filesystem_label.setter
    def initial_filesystem_label(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "initial_filesystem_label", value)

    @_builtins.property
    @pulumi.getter(name="initialFilesystemType")
    def initial_filesystem_type(self) -> Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]]:
        """
        Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        """
        return pulumi.get(self, "initial_filesystem_type")

    @initial_filesystem_type.setter
    def initial_filesystem_type(self, value: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]]):
        pulumi.set(self, "initial_filesystem_type", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[Union[_builtins.str, 'Region']]]:
        """
        The region that the block storage volume will be created in.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[Union[_builtins.str, 'Region']]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def size(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The size of the block storage volume in GiB. If updated, can only be expanded.
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "size", value)

    @_builtins.property
    @pulumi.getter(name="snapshotId")
    def snapshot_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        """
        return pulumi.get(self, "snapshot_id")

    @snapshot_id.setter
    def snapshot_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "snapshot_id", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of the tags to be applied to this Volume.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter(name="volumeUrn")
    def volume_urn(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The uniform resource name for the volume.
        """
        return pulumi.get(self, "volume_urn")

    @volume_urn.setter
    def volume_urn(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "volume_urn", value)


@pulumi.type_token("digitalocean:index/volume:Volume")
class Volume(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 filesystem_type: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_type: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
                 size: Optional[pulumi.Input[_builtins.int]] = None,
                 snapshot_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        """
        Provides a DigitalOcean Block Storage volume which can be attached to a Droplet in order to provide expanded storage.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        foobar = digitalocean.Volume("foobar",
            region=digitalocean.Region.NYC1,
            name="baz",
            size=100,
            initial_filesystem_type=digitalocean.FileSystemType.EXT4,
            description="an example volume")
        foobar_droplet = digitalocean.Droplet("foobar",
            name="baz",
            size=digitalocean.DropletSlug.DROPLET_S1_VCPU1_GB,
            image="ubuntu-18-04-x64",
            region=digitalocean.Region.NYC1)
        foobar_volume_attachment = digitalocean.VolumeAttachment("foobar",
            droplet_id=foobar_droplet.id,
            volume_id=foobar.id)
        ```

        You can also create a volume from an existing snapshot.

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        foobar = digitalocean.get_volume_snapshot(name="baz")
        foobar_volume = digitalocean.Volume("foobar",
            region=digitalocean.Region.LON1,
            name="foo",
            size=foobar.min_disk_size,
            snapshot_id=foobar.id)
        ```

        ## Import

        Volumes can be imported using the `volume id`, e.g.

        ```sh
        $ pulumi import digitalocean:index/volume:Volume volume 506f78a4-e098-11e5-ad9f-000f53306ae1
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] description: A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        :param pulumi.Input[_builtins.str] filesystem_type: Filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] initial_filesystem_label: Initial filesystem label for the block storage volume.
        :param pulumi.Input[Union[_builtins.str, 'FileSystemType']] initial_filesystem_type: Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] name: A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: The region that the block storage volume will be created in.
        :param pulumi.Input[_builtins.int] size: The size of the block storage volume in GiB. If updated, can only be expanded.
        :param pulumi.Input[_builtins.str] snapshot_id: The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of the tags to be applied to this Volume.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: VolumeArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a DigitalOcean Block Storage volume which can be attached to a Droplet in order to provide expanded storage.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        foobar = digitalocean.Volume("foobar",
            region=digitalocean.Region.NYC1,
            name="baz",
            size=100,
            initial_filesystem_type=digitalocean.FileSystemType.EXT4,
            description="an example volume")
        foobar_droplet = digitalocean.Droplet("foobar",
            name="baz",
            size=digitalocean.DropletSlug.DROPLET_S1_VCPU1_GB,
            image="ubuntu-18-04-x64",
            region=digitalocean.Region.NYC1)
        foobar_volume_attachment = digitalocean.VolumeAttachment("foobar",
            droplet_id=foobar_droplet.id,
            volume_id=foobar.id)
        ```

        You can also create a volume from an existing snapshot.

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        foobar = digitalocean.get_volume_snapshot(name="baz")
        foobar_volume = digitalocean.Volume("foobar",
            region=digitalocean.Region.LON1,
            name="foo",
            size=foobar.min_disk_size,
            snapshot_id=foobar.id)
        ```

        ## Import

        Volumes can be imported using the `volume id`, e.g.

        ```sh
        $ pulumi import digitalocean:index/volume:Volume volume 506f78a4-e098-11e5-ad9f-000f53306ae1
        ```

        :param str resource_name: The name of the resource.
        :param VolumeArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(VolumeArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 filesystem_type: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
                 initial_filesystem_type: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
                 size: Optional[pulumi.Input[_builtins.int]] = None,
                 snapshot_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = VolumeArgs.__new__(VolumeArgs)

            __props__.__dict__["description"] = description
            __props__.__dict__["filesystem_type"] = filesystem_type
            __props__.__dict__["initial_filesystem_label"] = initial_filesystem_label
            __props__.__dict__["initial_filesystem_type"] = initial_filesystem_type
            __props__.__dict__["name"] = name
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
            if size is None and not opts.urn:
                raise TypeError("Missing required property 'size'")
            __props__.__dict__["size"] = size
            __props__.__dict__["snapshot_id"] = snapshot_id
            __props__.__dict__["tags"] = tags
            __props__.__dict__["droplet_ids"] = None
            __props__.__dict__["filesystem_label"] = None
            __props__.__dict__["volume_urn"] = None
        super(Volume, __self__).__init__(
            'digitalocean:index/volume:Volume',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[_builtins.str]] = None,
            droplet_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
            filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
            filesystem_type: Optional[pulumi.Input[_builtins.str]] = None,
            initial_filesystem_label: Optional[pulumi.Input[_builtins.str]] = None,
            initial_filesystem_type: Optional[pulumi.Input[Union[_builtins.str, 'FileSystemType']]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
            size: Optional[pulumi.Input[_builtins.int]] = None,
            snapshot_id: Optional[pulumi.Input[_builtins.str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            volume_urn: Optional[pulumi.Input[_builtins.str]] = None) -> 'Volume':
        """
        Get an existing Volume resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] description: A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] droplet_ids: A list of associated droplet ids.
        :param pulumi.Input[_builtins.str] filesystem_label: Filesystem label for the block storage volume.
        :param pulumi.Input[_builtins.str] filesystem_type: Filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] initial_filesystem_label: Initial filesystem label for the block storage volume.
        :param pulumi.Input[Union[_builtins.str, 'FileSystemType']] initial_filesystem_type: Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        :param pulumi.Input[_builtins.str] name: A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: The region that the block storage volume will be created in.
        :param pulumi.Input[_builtins.int] size: The size of the block storage volume in GiB. If updated, can only be expanded.
        :param pulumi.Input[_builtins.str] snapshot_id: The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of the tags to be applied to this Volume.
        :param pulumi.Input[_builtins.str] volume_urn: The uniform resource name for the volume.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _VolumeState.__new__(_VolumeState)

        __props__.__dict__["description"] = description
        __props__.__dict__["droplet_ids"] = droplet_ids
        __props__.__dict__["filesystem_label"] = filesystem_label
        __props__.__dict__["filesystem_type"] = filesystem_type
        __props__.__dict__["initial_filesystem_label"] = initial_filesystem_label
        __props__.__dict__["initial_filesystem_type"] = initial_filesystem_type
        __props__.__dict__["name"] = name
        __props__.__dict__["region"] = region
        __props__.__dict__["size"] = size
        __props__.__dict__["snapshot_id"] = snapshot_id
        __props__.__dict__["tags"] = tags
        __props__.__dict__["volume_urn"] = volume_urn
        return Volume(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        A free-form text field up to a limit of 1024 bytes to describe a block storage volume.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="dropletIds")
    def droplet_ids(self) -> pulumi.Output[Sequence[_builtins.int]]:
        """
        A list of associated droplet ids.
        """
        return pulumi.get(self, "droplet_ids")

    @_builtins.property
    @pulumi.getter(name="filesystemLabel")
    def filesystem_label(self) -> pulumi.Output[_builtins.str]:
        """
        Filesystem label for the block storage volume.
        """
        return pulumi.get(self, "filesystem_label")

    @_builtins.property
    @pulumi.getter(name="filesystemType")
    @_utilities.deprecated("""This fields functionality has been replaced by `initial_filesystem_type`. The property will still remain as a computed attribute representing the current volumes filesystem type.""")
    def filesystem_type(self) -> pulumi.Output[_builtins.str]:
        """
        Filesystem type (`xfs` or `ext4`) for the block storage volume.
        """
        return pulumi.get(self, "filesystem_type")

    @_builtins.property
    @pulumi.getter(name="initialFilesystemLabel")
    def initial_filesystem_label(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Initial filesystem label for the block storage volume.
        """
        return pulumi.get(self, "initial_filesystem_label")

    @_builtins.property
    @pulumi.getter(name="initialFilesystemType")
    def initial_filesystem_type(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Initial filesystem type (`xfs` or `ext4`) for the block storage volume.
        """
        return pulumi.get(self, "initial_filesystem_type")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        A name for the block storage volume. Must be lowercase and be composed only of numbers, letters and "-", up to a limit of 64 characters. The name must begin with a letter.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region that the block storage volume will be created in.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def size(self) -> pulumi.Output[_builtins.int]:
        """
        The size of the block storage volume in GiB. If updated, can only be expanded.
        """
        return pulumi.get(self, "size")

    @_builtins.property
    @pulumi.getter(name="snapshotId")
    def snapshot_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The ID of an existing volume snapshot from which the new volume will be created. If supplied, the region and size will be limited on creation to that of the referenced snapshot
        """
        return pulumi.get(self, "snapshot_id")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        A list of the tags to be applied to this Volume.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="volumeUrn")
    def volume_urn(self) -> pulumi.Output[_builtins.str]:
        """
        The uniform resource name for the volume.
        """
        return pulumi.get(self, "volume_urn")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['DatabaseFirewallArgs', 'DatabaseFirewall']

@pulumi.input_type
class DatabaseFirewallArgs:
    def __init__(__self__, *,
                 cluster_id: pulumi.Input[_builtins.str],
                 rules: pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]]):
        """
        The set of arguments for constructing a DatabaseFirewall resource.
        :param pulumi.Input[_builtins.str] cluster_id: The ID of the target database cluster.
        :param pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]] rules: A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        pulumi.set(__self__, "cluster_id", cluster_id)
        pulumi.set(__self__, "rules", rules)

    @_builtins.property
    @pulumi.getter(name="clusterId")
    def cluster_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the target database cluster.
        """
        return pulumi.get(self, "cluster_id")

    @cluster_id.setter
    def cluster_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "cluster_id", value)

    @_builtins.property
    @pulumi.getter
    def rules(self) -> pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]]:
        """
        A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        return pulumi.get(self, "rules")

    @rules.setter
    def rules(self, value: pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]]):
        pulumi.set(self, "rules", value)


@pulumi.input_type
class _DatabaseFirewallState:
    def __init__(__self__, *,
                 cluster_id: Optional[pulumi.Input[_builtins.str]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]]] = None):
        """
        Input properties used for looking up and filtering DatabaseFirewall resources.
        :param pulumi.Input[_builtins.str] cluster_id: The ID of the target database cluster.
        :param pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]] rules: A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        if cluster_id is not None:
            pulumi.set(__self__, "cluster_id", cluster_id)
        if rules is not None:
            pulumi.set(__self__, "rules", rules)

    @_builtins.property
    @pulumi.getter(name="clusterId")
    def cluster_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the target database cluster.
        """
        return pulumi.get(self, "cluster_id")

    @cluster_id.setter
    def cluster_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cluster_id", value)

    @_builtins.property
    @pulumi.getter
    def rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]]]:
        """
        A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        return pulumi.get(self, "rules")

    @rules.setter
    def rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseFirewallRuleArgs']]]]):
        pulumi.set(self, "rules", value)


@pulumi.type_token("digitalocean:index/databaseFirewall:DatabaseFirewall")
class DatabaseFirewall(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cluster_id: Optional[pulumi.Input[_builtins.str]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseFirewallRuleArgs', 'DatabaseFirewallRuleArgsDict']]]]] = None,
                 __props__=None):
        """
        Provides a DigitalOcean database firewall resource allowing you to restrict
        connections to your database to trusted sources. You may limit connections to
        specific Droplets, Kubernetes clusters, or IP addresses.

        ## Example Usage

        ### Create a new database firewall allowing multiple IP addresses

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        example_fw = digitalocean.DatabaseFirewall("example-fw",
            cluster_id=postgres_example.id,
            rules=[
                {
                    "type": "ip_addr",
                    "value": "192.168.1.1",
                },
                {
                    "type": "ip_addr",
                    "value": "192.0.2.0",
                },
            ])
        ```

        ### Create a new database firewall allowing a Droplet

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        web = digitalocean.Droplet("web",
            name="web-01",
            size=digitalocean.DropletSlug.DROPLET_S1_VCPU1_GB,
            image="ubuntu-22-04-x64",
            region=digitalocean.Region.NYC3)
        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        example_fw = digitalocean.DatabaseFirewall("example-fw",
            cluster_id=postgres_example.id,
            rules=[{
                "type": "droplet",
                "value": web.id,
            }])
        ```

        ### Create a new database firewall for a database replica

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        replica_example = digitalocean.DatabaseReplica("replica-example",
            cluster_id=postgres_example.id,
            name="replica-example",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1)
        # Create firewall rule for database replica
        example_fw = digitalocean.DatabaseFirewall("example-fw",
            cluster_id=replica_example.uuid,
            rules=[{
                "type": "ip_addr",
                "value": "192.168.1.1",
            }])
        ```

        ## Import

        Database firewalls can be imported using the `id` of the target database cluster

        For example:

        ```sh
        $ pulumi import digitalocean:index/databaseFirewall:DatabaseFirewall example-fw 5f55c6cd-863b-4907-99b8-7e09b0275d54
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cluster_id: The ID of the target database cluster.
        :param pulumi.Input[Sequence[pulumi.Input[Union['DatabaseFirewallRuleArgs', 'DatabaseFirewallRuleArgsDict']]]] rules: A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: DatabaseFirewallArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a DigitalOcean database firewall resource allowing you to restrict
        connections to your database to trusted sources. You may limit connections to
        specific Droplets, Kubernetes clusters, or IP addresses.

        ## Example Usage

        ### Create a new database firewall allowing multiple IP addresses

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        example_fw = digitalocean.DatabaseFirewall("example-fw",
            cluster_id=postgres_example.id,
            rules=[
                {
                    "type": "ip_addr",
                    "value": "192.168.1.1",
                },
                {
                    "type": "ip_addr",
                    "value": "192.0.2.0",
                },
            ])
        ```

        ### Create a new database firewall allowing a Droplet

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        web = digitalocean.Droplet("web",
            name="web-01",
            size=digitalocean.DropletSlug.DROPLET_S1_VCPU1_GB,
            image="ubuntu-22-04-x64",
            region=digitalocean.Region.NYC3)
        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        example_fw = digitalocean.DatabaseFirewall("example-fw",
            cluster_id=postgres_example.id,
            rules=[{
                "type": "droplet",
                "value": web.id,
            }])
        ```

        ### Create a new database firewall for a database replica

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        replica_example = digitalocean.DatabaseReplica("replica-example",
            cluster_id=postgres_example.id,
            name="replica-example",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1)
        # Create firewall rule for database replica
        example_fw = digitalocean.DatabaseFirewall("example-fw",
            cluster_id=replica_example.uuid,
            rules=[{
                "type": "ip_addr",
                "value": "192.168.1.1",
            }])
        ```

        ## Import

        Database firewalls can be imported using the `id` of the target database cluster

        For example:

        ```sh
        $ pulumi import digitalocean:index/databaseFirewall:DatabaseFirewall example-fw 5f55c6cd-863b-4907-99b8-7e09b0275d54
        ```

        :param str resource_name: The name of the resource.
        :param DatabaseFirewallArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(DatabaseFirewallArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cluster_id: Optional[pulumi.Input[_builtins.str]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseFirewallRuleArgs', 'DatabaseFirewallRuleArgsDict']]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = DatabaseFirewallArgs.__new__(DatabaseFirewallArgs)

            if cluster_id is None and not opts.urn:
                raise TypeError("Missing required property 'cluster_id'")
            __props__.__dict__["cluster_id"] = cluster_id
            if rules is None and not opts.urn:
                raise TypeError("Missing required property 'rules'")
            __props__.__dict__["rules"] = rules
        super(DatabaseFirewall, __self__).__init__(
            'digitalocean:index/databaseFirewall:DatabaseFirewall',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            cluster_id: Optional[pulumi.Input[_builtins.str]] = None,
            rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseFirewallRuleArgs', 'DatabaseFirewallRuleArgsDict']]]]] = None) -> 'DatabaseFirewall':
        """
        Get an existing DatabaseFirewall resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cluster_id: The ID of the target database cluster.
        :param pulumi.Input[Sequence[pulumi.Input[Union['DatabaseFirewallRuleArgs', 'DatabaseFirewallRuleArgsDict']]]] rules: A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _DatabaseFirewallState.__new__(_DatabaseFirewallState)

        __props__.__dict__["cluster_id"] = cluster_id
        __props__.__dict__["rules"] = rules
        return DatabaseFirewall(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="clusterId")
    def cluster_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the target database cluster.
        """
        return pulumi.get(self, "cluster_id")

    @_builtins.property
    @pulumi.getter
    def rules(self) -> pulumi.Output[Sequence['outputs.DatabaseFirewallRule']]:
        """
        A rule specifying a resource allowed to access the database cluster. The following arguments must be specified:
        """
        return pulumi.get(self, "rules")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetSpacesBucketsResult',
    'AwaitableGetSpacesBucketsResult',
    'get_spaces_buckets',
    'get_spaces_buckets_output',
]

@pulumi.output_type
class GetSpacesBucketsResult:
    """
    A collection of values returned by getSpacesBuckets.
    """
    def __init__(__self__, buckets=None, filters=None, id=None, sorts=None):
        if buckets and not isinstance(buckets, list):
            raise TypeError("Expected argument 'buckets' to be a list")
        pulumi.set(__self__, "buckets", buckets)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if sorts and not isinstance(sorts, list):
            raise TypeError("Expected argument 'sorts' to be a list")
        pulumi.set(__self__, "sorts", sorts)

    @_builtins.property
    @pulumi.getter
    def buckets(self) -> Sequence['outputs.GetSpacesBucketsBucketResult']:
        """
        A list of Spaces buckets satisfying any `filter` and `sort` criteria. Each bucket has the following attributes:
        """
        return pulumi.get(self, "buckets")

    @_builtins.property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetSpacesBucketsFilterResult']]:
        return pulumi.get(self, "filters")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def sorts(self) -> Optional[Sequence['outputs.GetSpacesBucketsSortResult']]:
        return pulumi.get(self, "sorts")


class AwaitableGetSpacesBucketsResult(GetSpacesBucketsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetSpacesBucketsResult(
            buckets=self.buckets,
            filters=self.filters,
            id=self.id,
            sorts=self.sorts)


def get_spaces_buckets(filters: Optional[Sequence[Union['GetSpacesBucketsFilterArgs', 'GetSpacesBucketsFilterArgsDict']]] = None,
                       sorts: Optional[Sequence[Union['GetSpacesBucketsSortArgs', 'GetSpacesBucketsSortArgsDict']]] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetSpacesBucketsResult:
    """
    Get information on Spaces buckets for use in other resources, with the ability to filter and sort the results.
    If no filters are specified, all Spaces buckets will be returned.

    Note: You can use the `SpacesBucket` data source to
    obtain metadata about a single bucket if you already know its `name` and `region`.

    ## Example Usage

    Use the `filter` block with a `key` string and `values` list to filter buckets.

    Get all buckets in a region:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    nyc3 = digitalocean.get_spaces_buckets(filters=[{
        "key": "region",
        "values": ["nyc3"],
    }])
    ```
    You can sort the results as well:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    nyc3 = digitalocean.get_spaces_buckets(filters=[{
            "key": "region",
            "values": ["nyc3"],
        }],
        sorts=[{
            "key": "name",
            "direction": "desc",
        }])
    ```


    :param Sequence[Union['GetSpacesBucketsFilterArgs', 'GetSpacesBucketsFilterArgsDict']] filters: Filter the results.
           The `filter` block is documented below.
    :param Sequence[Union['GetSpacesBucketsSortArgs', 'GetSpacesBucketsSortArgsDict']] sorts: Sort the results.
           The `sort` block is documented below.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['sorts'] = sorts
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getSpacesBuckets:getSpacesBuckets', __args__, opts=opts, typ=GetSpacesBucketsResult).value

    return AwaitableGetSpacesBucketsResult(
        buckets=pulumi.get(__ret__, 'buckets'),
        filters=pulumi.get(__ret__, 'filters'),
        id=pulumi.get(__ret__, 'id'),
        sorts=pulumi.get(__ret__, 'sorts'))
def get_spaces_buckets_output(filters: Optional[pulumi.Input[Optional[Sequence[Union['GetSpacesBucketsFilterArgs', 'GetSpacesBucketsFilterArgsDict']]]]] = None,
                              sorts: Optional[pulumi.Input[Optional[Sequence[Union['GetSpacesBucketsSortArgs', 'GetSpacesBucketsSortArgsDict']]]]] = None,
                              opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetSpacesBucketsResult]:
    """
    Get information on Spaces buckets for use in other resources, with the ability to filter and sort the results.
    If no filters are specified, all Spaces buckets will be returned.

    Note: You can use the `SpacesBucket` data source to
    obtain metadata about a single bucket if you already know its `name` and `region`.

    ## Example Usage

    Use the `filter` block with a `key` string and `values` list to filter buckets.

    Get all buckets in a region:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    nyc3 = digitalocean.get_spaces_buckets(filters=[{
        "key": "region",
        "values": ["nyc3"],
    }])
    ```
    You can sort the results as well:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    nyc3 = digitalocean.get_spaces_buckets(filters=[{
            "key": "region",
            "values": ["nyc3"],
        }],
        sorts=[{
            "key": "name",
            "direction": "desc",
        }])
    ```


    :param Sequence[Union['GetSpacesBucketsFilterArgs', 'GetSpacesBucketsFilterArgsDict']] filters: Filter the results.
           The `filter` block is documented below.
    :param Sequence[Union['GetSpacesBucketsSortArgs', 'GetSpacesBucketsSortArgsDict']] sorts: Sort the results.
           The `sort` block is documented below.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['sorts'] = sorts
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getSpacesBuckets:getSpacesBuckets', __args__, opts=opts, typ=GetSpacesBucketsResult)
    return __ret__.apply(lambda __response__: GetSpacesBucketsResult(
        buckets=pulumi.get(__response__, 'buckets'),
        filters=pulumi.get(__response__, 'filters'),
        id=pulumi.get(__response__, 'id'),
        sorts=pulumi.get(__response__, 'sorts')))
